#! /bin/sh
DEFAULTFREQ=60   # Default frequency is 60 minutes ( once per hour)
#Mount sysfs if it is not mounted
/bin/mount | grep -q sysfs
if [ $? -ne 0 ]; then
	/bin/mount -t sysfs none /sys
fi
runno=1
if [ ! -d /var/log/mstatdir ]; then
	/bin/mkdir /var/log/mstatdir
fi
/bin/cat /proc/cmdline | /bin/grep -q MSTATFREQ
if [ $? -eq 0 ]; then
    MEMDUMPFREQ=`/bin/cat /proc/cmdline | /bin/sed 's:^.*MSTATFREQ=\([0-9]\+\).*:\1:'`
    if [ -z "${MEMDUMPFREQ}" ] || [ -z "${MEMDUMPFREQ+xxx}" ]; then
	    MEMDUMPFREQ=$DEFAULTFREQ;
    fi
    echo "Using command line value of $MEMDUMPFREQ minutes to dump memory status" >> /var/log/mstatdir/memorystatus
else
    MEMDUMPFREQ=$DEFAULTFREQ;
    echo "Using Default value of $MEMDUMPFREQ minutes to dump memory status" >> /var/log/mstatdir/memorystatus
fi
#if the frequency of data collection is more than once every 10 minutes, we rotate the logs every hour
if [ "${MEMDUMPFREQ}" -lt 10 ]; then
    if [ -f /etc/cron.daily/logrotate ]; then 
	/bin/mv /etc/cron.daily/logrotate /etc/cron.hourly/logrotate
    fi
    /bin/sync;
    echo "Rotating the logs every hour due to user setting" >> /var/log/mstatdir/memorystatus
else 
    if [ -f /etc/cron.hourly/logrotate ]; then
	/bin/mv /etc/cron.hourly/logrotate /etc/cron.daily/logrotate
    fi
    /bin/sync;
fi
#while [ 1 ] ;
#do
#	echo "Running memorystatus daemon ";
	echo "Output of /proc/buddyinfo: `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	echo "===============================================" >> /var/log/mstatdir/memorystatus
	cat /proc/buddyinfo >> /var/log/mstatdir/memorystatus
	echo "" >> /var/log/mstatdir/memorystatus
	export cachesize=`cat /proc/meminfo  | grep ^Cached | awk '{print $2}'` 
	if [ $cachesize -gt 450000 ]; then
		echo "Cache size more than 450 MB. Clearing it" >> /var/log/mstatdir/memorystatus
		echo "Output of /proc/meminfo: `date +%F--%T`" >> /var/log/mstatdir/memorystatus
		echo "=============================================" >> /var/log/mstatdir/memorystatus
		cat /proc/meminfo  >> /var/log/mstatdir/memorystatus
		echo 3 > /proc/sys/vm/drop_caches
	fi;
	echo "" >> /var/log/mstatdir/memorystatus
	echo "Output of /proc/meminfo: `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	echo "=============================================" >> /var/log/mstatdir/memorystatus
	cat /proc/meminfo  >> /var/log/mstatdir/memorystatus
	echo "" >> /var/log/mstatdir/memorystatus
	echo "Output of /bin/slabinfo (Top 20): `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	echo "=====================================================" >> /var/log/mstatdir/memorystatus
	/bin/slabinfo -S | head -20 >> /var/log/mstatdir/memorystatus
	echo "" >> /var/log/mstatdir/memorystatus
	echo "procrank_startrun_$runno" >> /var/log/mstatdir/memorystatus
	echo "Output of /bin/procrank: `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	echo "======================================================" >> /var/log/mstatdir/memorystatus
	/bin/procrank -R     >> /var/log/mstatdir/memorystatus
	echo "procrank_endrun_$runno" >> /var/log/mstatdir/memorystatus
	echo "" >> /var/log/mstatdir/memorystatus
	if [ -d "/proc/mlt" ]; then
	    echo "Output of /bin/mlt -s 20 (Top 20): `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	    echo "=======================================================" >> /var/log/mstatdir/memorystatus
	    /bin/mlt -s 20 >> /var/log/mstatdir/memorystatus
	    export memrows=`/bin/mlt -s 20 | awk '{if(NR > 9) print $0}' | head -n -16 | awk -f /usr/bin/awkmemstatus`;
	    if [ ! -z "${memrows}" ]; then
		echo "" >> /var/log/mstatdir/memorystatus
		echo "Rows $memrows  are more than threshold 50MB" >> /var/log/mstatdir/memorystatus ;
		for i in ${memrows}
		do 
			/bin/mlt -d $i  >> /var/log/mstatdir/memorystatus
			echo "" >> /var/log/mstatdir/memorystatus
		done;
	    fi;
	    echo "" >> /var/log/mstatdir/memorystatus
	    echo "Output of /bin/mlt -t 20 (Top 20): `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	    echo "=======================================================" >> /var/log/mstatdir/memorystatus
	    /bin/mlt -t 20 >> /var/log/mstatdir/memorystatus
	    export kmallocrows=`/bin/mlt -t 20 | awk '{if(NR > 9) print $0}' | head -n -16 | awk -f /usr/bin/awkkmalloccount`;
	fi

	if [ -d "/proc/mlt_kl" ]; then
	    echo "Output of /bin/mlt_kl -s 20 (Top 20): `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	    echo "=======================================================" >> /var/log/mstatdir/memorystatus
	    /bin/mlt_kl -s 20 >> /var/log/mstatdir/memorystatus
	    export memrows=`/bin/mlt_kl -s 20 | awk '{if(NR > 6) print $0}' | awk -f /usr/bin/awkmemstatus`;
	    if [ ! -z "${memrows}" ]; then
		echo "" >> /var/log/mstatdir/memorystatus
		echo "Rows $memrows  are more than threshold 50MB" >> /var/log/mstatdir/memorystatus ;
		for i in ${memrows}
		do 
			/bin/mlt_kl -d $i  >> /var/log/mstatdir/memorystatus
			echo "" >> /var/log/mstatdir/memorystatus
		done;
	    fi;
	    echo "" >> /var/log/mstatdir/memorystatus
	    echo "Output of /bin/mlt_kl -t 20 (Top 20): `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	    echo "=======================================================" >> /var/log/mstatdir/memorystatus
	    /bin/mlt_kl -t 20 >> /var/log/mstatdir/memorystatus
	    export kmallocrows=`/bin/mlt_kl -t 20 | awk '{if(NR > 6) print $0}' | awk -f /usr/bin/awkkmalloccount`;

	    if [ ! -z "${kmallocrows}" ]; then
		echo "" >> /var/log/mstatdir/memorystatus
		echo "Rows $kmallocrows  are more than threshold 20K" >> /var/log/mstatdir/memorystatus ;
		for i in ${kmallocrows}
		do 
			/bin/mlt_kl -d $i  >> /var/log/mstatdir/memorystatus
			echo "" >> /var/log/mstatdir/memorystatus
		done;
	    fi;
	fi

	if [ -d "/proc/mlt_vm" ]; then
	    echo "Output of /bin/mlt_vm -s 20 (Top 20): `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	    echo "=======================================================" >> /var/log/mstatdir/memorystatus
	    /bin/mlt_vm -s 20 >> /var/log/mstatdir/memorystatus
	    export memrows=`/bin/mlt_vm -s 20 | awk '{if(NR > 6) print $0}' | awk -f /usr/bin/awkmemstatus`;
	    if [ ! -z "${memrows}" ]; then
		echo "" >> /var/log/mstatdir/memorystatus
		echo "Rows $memrows  are more than threshold 50MB" >> /var/log/mstatdir/memorystatus ;
		for i in ${memrows}
		do 
			/bin/mlt_vm -d $i  >> /var/log/mstatdir/memorystatus
			echo "" >> /var/log/mstatdir/memorystatus
		done;
	    fi;
	    echo "" >> /var/log/mstatdir/memorystatus
	    echo "Output of /bin/mlt_vm -t 20 (Top 20): `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	    echo "=======================================================" >> /var/log/mstatdir/memorystatus
	    /bin/mlt_vm -t 20 >> /var/log/mstatdir/memorystatus
	    export kmallocrows=`/bin/mlt_vm -t 20 | awk '{if(NR > 6) print $0}' | awk -f /usr/bin/awkkmalloccount`;

	    if [ ! -z "${kmallocrows}" ]; then
		echo "" >> /var/log/mstatdir/memorystatus
		echo "Rows $kmallocrows  are more than threshold 20K" >> /var/log/mstatdir/memorystatus ;
		for i in ${kmallocrows}
		do 
			/bin/mlt_vm -d $i  >> /var/log/mstatdir/memorystatus
			echo "" >> /var/log/mstatdir/memorystatus
		done;
	    fi;
	fi
	echo "" >> /var/log/mstatdir/memorystatus
	echo "Ten big files in /tmp: `date +%F--%T`" >> /var/log/mstatdir/memorystatus
	echo "============================================" >> /var/log/mstatdir/memorystatus
	/bin/ls -lrS /tmp | tail -10  >> /var/log/mstatdir/memorystatus
	echo "" >> /var/log/mstatdir/memorystatus
	echo "-------------------------------------------------------------------------------" >> /var/log/mstatdir/memorystatus
	echo "" >> /var/log/mstatdir/memorystatus
	#sleep $((MEMDUMPFREQ * 60));
	#runno=$((runno + 1));
#done;
